#include "gl_window.h"
#include <stdio.h>
#include <stdlib.h>

gl_window :: gl_window(int x, int y, int h, int w, const char* label) :
Fl_Gl_Window(x, y, h, w, label)
{ 
	/* create the window */
	glutCreateWindow("");

	/* set double buffer, rga, depth sorted */
	mode(FL_DOUBLE | FL_RGB | FL_DEPTH);
	
	/* init clip planes */
	near_clip = 0.1;
	far_clip = 100.0;
	asp_ratio = h / w;

	rotx = 0; roty = 0; rotz = 0;

}

gl_window :: ~gl_window(void)
{
	// do nothing yet
}

void gl_window :: rotate(void)
{
	rotx < 360 ? (rand() % 10 > 5 ? rotx++ : rotx += 2) : rotx -= 360;
	roty < 360 ? (rand() % 10 > 5 ? roty++ : roty += 2) : roty -= 360;
	rotz < 360 ? (rand() % 10 > 5 ? rotz++ : rotz += 2) : rotz -= 360;

	redraw();

}

int gl_window :: handle(int e)
{

	// if we have a mouse button push
	if(e == FL_PUSH)
	{
		// check which button
		if(Fl :: event_state() == FL_BUTTON1)
		{
			redraw();
			return 1;
		}
	}
	// maybe it was the keyboard
	else
	if(e == FL_KEYBOARD)
	{
		// was it the a key?
		if(Fl :: event_key() == 'a') 
		{
			persp = !persp;
			redraw();
			return 1;
		}
	}
	
	// must have been something we don't care about
	
	return 0;
}

void gl_window :: draw(void)
{

	// give ourselves focus
	Fl :: focus(this);
	
	/* init hardware rendering */

	/* set the background color */
	glClearColor(0, 0, 0, 1);

	/* reset the buffer bits */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	/* drop to projection mode */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	/* set up the viewing frustum */
	if(persp)
		glFrustum(-asp_ratio * .05, asp_ratio * .05, -.05, .05,
			near_clip, far_clip);
	else
		glOrtho(-2.5, 2.5, -2.5, 2.5, near_clip, far_clip);

	/* go back to world coordinates */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	/* move out */
	glTranslatef(0.0f, 0.0f, -5.0f);

	/* rotate object */
	glRotatef(rotx, 1, 0, 0);
	glRotatef(roty, 0, 1, 0);
	glRotatef(rotz, 0, 0, 1);

	/* set the drawing color */
	glColor3f(.5, .6, .7);

	/* draw something */
	for(float f = 2.5; f >= .5; f -= 1)
		glutWireCube(f);
		
	/* swap buffers */
	glutSwapBuffers();

}
