#include "gl_window.h"
#include <stdio.h>
#include <stdlib.h>

gl_window :: gl_window(int x, int y, int h, int w, const char* label) :
Fl_Gl_Window(x, y, h, w, label)
{ 
	/* create the window */
	glutCreateWindow("");

	/* set double buffer, rga, depth sorted */
	mode(FL_DOUBLE | FL_RGB | FL_DEPTH);
	
	/* init clip planes */
	near_clip = 0.1;
	far_clip = 150.0;
	asp_ratio = h / w;

	/* rotation increments */
	rotx = 0; roty = 30; rotz = 149;

}

gl_window :: ~gl_window(void)
{
	// do nothing yet
}

void gl_window :: rotate(void)
{
	rotx < 360 ? (rand() % 10 > 5 ? rotx++ : rotx += 2) : rotx -= 360;
	roty < 360 ? (rand() % 10 > 5 ? roty+= 3 : roty += 2) : roty -= 360;
	//rotz < 360 ? (rand() % 10 > 5 ? rotz++ : rotz += 2) : rotz -= 360;

	redraw();

}

int gl_window :: handle(int e)
{

	// if we have a mouse button push
	if(e == FL_PUSH)
	{
		// check which button
		if(Fl :: event_state() == FL_BUTTON1)
		{
			redraw();
			return 1;
		}
	}
	// maybe it was the keyboard
	else
	if(e == FL_KEYBOARD)
	{
		// was it the a key?
		if(Fl :: event_key() == 'a') 
		{
			redraw();
			return 1;
		}
	}
	
	// must have been something we don't care about
	
	return 0;
}

void gl_window :: draw(void)
{

	/*
	// basic light
	static GLfloat light_pos[] = {0, 2, 5, 0};
	static GLfloat diffuse[] = {1, 0, 0, 1};
	static GLfloat specular[] = {1, 1, 1, .2};
	static GLfloat mat_specular[] = {1, 1, 1, 0};
	static GLfloat mat_diffuse[] = {0, 0, 0, 0};
	*/
	
	// give ourselves focus
	Fl :: focus(this);
	
	/* init hardware rendering */

	/* set the background color */
	glClearColor(0, 0, 0, 1);

	/* reset the buffer bits */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	/* drop to projection mode */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	/* set up the viewing frustum */
	glFrustum(-asp_ratio * .05, asp_ratio * .05, -.05, .05,
	near_clip, far_clip);

	/* go back to world coordinates */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	/* enable checks */
	glEnable(GL_DEPTH);
	glEnable(GL_DEPTH_TEST);

	/* enable lighting */
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	/*
	// lighting properties
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
	
	// material properties
	glMaterialf(GL_FRONT, GL_SHININESS, 1);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
	*/
	
	// Gouraud Shading
	glShadeModel(GL_SMOOTH);

	/* move out */
	glTranslatef(0.0f, 0.0f, -5.0f);

	/* draw floor */
	glBegin(GL_QUADS);
		glNormal3f(0, 1, 0);
		glVertex3d(-5, -2, -5);
		glVertex3d(5, -2, -5);
		glVertex3d(5, -3, 5);
		glVertex3d(-5, -3, 5);
	glEnd();

	/* rotate object */
	glRotatef(rotx, 1, 0, 0);
	glRotatef(roty, 0, 1, 0);
	glRotatef(rotz, 0, 0, 1);

	/* draw something */
	//glutSolidCube(1.5);
	//glutSolidSphere(1, 50, 50);
	//glutSolidTeapot(1);
	glutSolidTorus(.4, .75, 50, 50);
		
	/* swap buffers */
	glutSwapBuffers();

}
