#include "gl_window.h"

#define PI 3.14159

void gl_window :: drawSphere(GLdouble m_dRadius, GLint m_iSlices, GLint m_iStacks) 
{

  GLint i = 0;
  GLint slices = m_iSlices;  

  GLfloat ** unitCircle = new GLfloat * [ slices+1];
  for( i = 0; i < slices+1; ++i )
    unitCircle[ i ] = new GLfloat[3];

  double angle = 0.0;
  double angleStep = 2.0 * PI / (double)slices;
  double dist = (float) 1 / m_iSlices;
  double dist2 = (float) 1 / m_iStacks;

  // circle of radius 1, traced counterclockwise
  for( i = 0; i < slices+1; ++i )
  {
    unitCircle[i][0] = (GLfloat)cos( angle );
    unitCircle[i][1] = (GLfloat)sin( angle );
    unitCircle[i][2] = 0.0;
    angle += angleStep;
  }

  // scaling
  // draw sphere
  // ===========
  // moving along z axis, from positive, to negative

  GLint   stack = 1;
  GLfloat stackAngle = (GLfloat)PI / (GLfloat)m_iStacks;

  GLfloat radius1 = 0.0;
  GLfloat radius2 = (GLfloat)sin( stack * stackAngle );

  GLvector va, vb, vc, vd; //na, nb, nc;
  GLvector vtemp;
  GLfloat z1 = 1.0;
  GLfloat z2 = (GLfloat)cos( stack * stackAngle );

  //GLvector v( 0, 0, 1 );
  //GLvector n( 0, 0, 1 );

  GLvector oldv, oldn;

  // "north" pole, 1st stack
  // -----------------------

  oldv[0] = radius2 * unitCircle[0][0];
  oldv[1] = radius2 * unitCircle[0][1];
  oldv[2] = z2;
  oldn = oldv;


 
   // sphere is rendered as a stack of quads
   // each quad is like a square (logitude/latitude) on a map
   // even though the top of the sphere is actually a triangles, it 
   // should be drawn as a quads (where two vertices have the same location) 
   // at the top
 
  // if you look at a map "stacks" strips of rectangles from top to bottom 
  // "strips" are the rectangles from right to left on a map

	// z2 starts at the top of the sphere
	z2=1;
	// the radius at the top is zero (all the top quads are at the north pole like a map)
	radius2=0;
	
   glBegin( GL_QUADS );
  // for each stack of quads (we start with the upper left corner of a map
   for( stack = 1; stack <m_iStacks+1; ++stack )
  {
    radius1 = radius2;
    radius2 = (GLfloat)sin( stack * stackAngle );

    z1 = z2;
    z2 = (GLfloat)cos( stack * stackAngle );

    va[0] = radius1 * unitCircle[0][0];
    va[1] = radius1 * unitCircle[0][1];
    va[2] = z1;

    vc[0] = radius2 * unitCircle[0][0];
    vc[1] = radius2 * unitCircle[0][1];
    vc[2] = z2;
	// for each quad going around the sphere
       // going east to west
    for( i = 0; i < slices; ++i )
    {
      glNormal3fv( va.c );
	 // scale by the radius
	 vtemp=va*m_dRadius;
	   glTexCoord2f(((float) i/m_iSlices),((float)(stack-1)/m_iStacks));
       glVertex3fv( vtemp.c );
	
      vb[0] = radius1 * unitCircle[i+1][0];
      vb[1] = radius1 * unitCircle[i+1][1];
      vb[2] = z1;

      glNormal3fv( vb.c );
	 // scale by the radius	 
	 vtemp=vb*m_dRadius;
	   glTexCoord2f(((float )i/m_iSlices)+dist,((float)(stack-1)/m_iStacks));
      glVertex3fv( vtemp.c );

      vd[0] = radius2 * unitCircle[i+1][0];
      vd[1] = radius2 * unitCircle[i+1][1];
      vd[2] = z2;

      glNormal3fv( vd.c );
	 // scale by the radius	 
	 vtemp=vd*m_dRadius;
	   glTexCoord2f(((float)i/m_iSlices)+dist,((float)(stack-1)/m_iStacks)+dist2);
      glVertex3fv( vtemp.c );

      glNormal3fv( vc.c );
	 // scale by the radius	 
	 vtemp=vc*m_dRadius;
	   glTexCoord2f(((float)i/m_iSlices),((float)(stack-1)/m_iStacks)+dist2);
      glVertex3fv( vtemp.c );
      va = vb;
      vc = vd;
    }
    
  }
  glEnd();
  
  for( i = 0; i < slices; ++i )
    delete [] unitCircle[i];
  delete [] unitCircle;
} // renderLit


