#include "gl_window.h"
#include "GLTypes.h"

void gl_window :: loadPPM(const char * filename, int id)
{

  	FILE *infile = NULL;
	int width = 0,height = 0, maxValue = 0;
	char buf[70];
	
	//Open PPM file
	infile = fopen(filename,"rb");

	//check if it is correct format
	fscanf(infile,"%s",buf);

	if (strcmp("P6",buf) != 0)
		printf ("Improper Format\n");
	else
	{
		//go to next line
		fscanf(infile,"%s",buf);
		while(isspace(buf[0]))
		{
			fscanf(infile,"%s",buf);
		}

		//check for comment
		if (buf[0] == '#')
		{
		  //printf("%s",buf);
			while(buf[0]!= '\n')
			{
				fscanf(infile,"%c",&buf[0]);
			}

			fscanf(infile,"%s",buf);
		}

		//Set width of map
		width = atoi(buf);

		//Set height of map
		fscanf(infile,"%s",buf);
		height = atoi(buf);

		//create array for colors
		_texture[id-1] = new unsigned char[width*height*3];

		//Set max value for intensity calculations
		fscanf(infile,"%s",buf);
		maxValue = atoi(buf);
   		fscanf(infile,"%c",buf);

		//Read texture map
		for (int i = 0; i < height; i++)
		   	fread((void *) &_texture[(id-1)]
				[(3 * ((width * (height - 1)) - i * width))],
			 	 3, width, infile);
	    
	}

	fclose(infile);
	tex_width[id-1] = width;
	tex_height[id-1] = height;

	loadTex(id);

}

void gl_window :: loadTex(int id)
{
	// woo! send to texture memory
	glBindTexture(GL_TEXTURE_2D, id-1);
	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	// texture options
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

	glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, tex_width[id-1], tex_height[id-1], 0,
					GL_RGB, GL_UNSIGNED_BYTE, _texture[id-1]);

	glEnable(GL_TEXTURE_2D);

}

void gl_window :: drawSphere(GLdouble m_dRadius, GLint m_iSlices, GLint m_iStacks) 
{

  GLint i = 0;
  GLint slices = m_iSlices;  

  GLfloat ** unitCircle = new GLfloat * [ slices+1];
  for( i = 0; i < slices+1; ++i )
    unitCircle[ i ] = new GLfloat[3];

  double angle = 0.0;
  double angleStep = 2.0 * M_PI / (double)slices;
  double dist = (float) 1 / m_iSlices;
  double dist2 = (float) 1 / m_iStacks;

  // circle of radius 1, traced counterclockwise
  for( i = 0; i < slices+1; ++i )
  {
    unitCircle[i][0] = (GLfloat)cos( angle );
    unitCircle[i][1] = (GLfloat)sin( angle );
    unitCircle[i][2] = 0.0;
    angle += angleStep;
  }

  // scaling
  // draw sphere
  // ===========
  // moving along z axis, from positive, to negative

  GLint   stack = 1;
  GLfloat stackAngle = (GLfloat)M_PI / (GLfloat)m_iStacks;

  GLfloat radius1 = 0.0;
  GLfloat radius2 = (GLfloat)sin( stack * stackAngle );

  GLvector va, vb, vc, vd; //na, nb, nc;
  GLvector vtemp;
  GLfloat z1 = 1.0;
  GLfloat z2 = (GLfloat)cos( stack * stackAngle );

  //GLvector v( 0, 0, 1 );
  //GLvector n( 0, 0, 1 );

  GLvector oldv, oldn;

  // "north" pole, 1st stack
  // -----------------------

  oldv[0] = radius2 * unitCircle[0][0];
  oldv[1] = radius2 * unitCircle[0][1];
  oldv[2] = z2;
  oldn = oldv;


 
   // sphere is rendered as a stack of quads
   // each quad is like a square (logitude/latitude) on a map
   // even though the top of the sphere is actually a triangles, it 
   // should be drawn as a quads (where two vertices have the same location) 
   // at the top
 
  // if you look at a map "stacks" strips of rectangles from top to bottom 
  // "strips" are the rectangles from right to left on a map

	// z2 starts at the top of the sphere
	z2=1;
	// the radius at the top is zero (all the top quads are at the north pole like a map)
	radius2=0;
	
   glBegin( GL_QUADS );
  // for each stack of quads (we start with the upper left corner of a map
   for( stack = 1; stack <m_iStacks+1; ++stack )
  {
    radius1 = radius2;
    radius2 = (GLfloat)sin( stack * stackAngle );

    z1 = z2;
    z2 = (GLfloat)cos( stack * stackAngle );

    va[0] = radius1 * unitCircle[0][0];
    va[1] = radius1 * unitCircle[0][1];
    va[2] = z1;

    vc[0] = radius2 * unitCircle[0][0];
    vc[1] = radius2 * unitCircle[0][1];
    vc[2] = z2;
	// for each quad going around the sphere
       // going east to west
    for( i = 0; i < slices; ++i )
    {
      glNormal3fv( va.c );
	 // scale by the radius
	 vtemp=va*m_dRadius;
	   glTexCoord2f(((float) i/m_iSlices),((float)(stack-1)/m_iStacks));
       glVertex3fv( vtemp.c );
	
      vb[0] = radius1 * unitCircle[i+1][0];
      vb[1] = radius1 * unitCircle[i+1][1];
      vb[2] = z1;

      glNormal3fv( vb.c );
	 // scale by the radius	 
	 vtemp=vb*m_dRadius;
	   glTexCoord2f(((float )i/m_iSlices)+dist,((float)(stack-1)/m_iStacks));
      glVertex3fv( vtemp.c );

      vd[0] = radius2 * unitCircle[i+1][0];
      vd[1] = radius2 * unitCircle[i+1][1];
      vd[2] = z2;

      glNormal3fv( vd.c );
	 // scale by the radius	 
	 vtemp=vd*m_dRadius;
	   glTexCoord2f(((float)i/m_iSlices)+dist,((float)(stack-1)/m_iStacks)+dist2);
      glVertex3fv( vtemp.c );

      glNormal3fv( vc.c );
	 // scale by the radius	 
	 vtemp=vc*m_dRadius;
	   glTexCoord2f(((float)i/m_iSlices),((float)(stack-1)/m_iStacks)+dist2);
      glVertex3fv( vtemp.c );
      va = vb;
      vc = vd;
    }
    
  }
  glEnd();
  
  for( i = 0; i < slices; ++i )
    delete [] unitCircle[i];
  delete [] unitCircle;
} // renderLit

void gl_window :: drawBox(double size)
{

	size /= 2;

	/* box */
	glBegin (GL_QUADS);
	
		// Front Face
		glNormal3d(0, 0, 1);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-size, -size,  size);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( size, -size,  size);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( size,  size,  size);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-size,  size,  size);
		// Back Face
		glNormal3d(0, 0, -1);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-size, -size, -size);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-size,  size, -size);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( size,  size, -size);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( size, -size, -size);
		// Top Face
		glNormal3d(0, 1, 0);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-size,  size, -size);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-size,  size,  size);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( size,  size,  size);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( size,  size, -size);
		// Bottom Face
		glNormal3d(0, -1, 0);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-size, -size, -size);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( size, -size, -size);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( size, -size,  size);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-size, -size,  size);
		// Right face
		glNormal3d(1, 0, 0);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( size, -size, -size);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( size,  size, -size);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( size,  size,  size);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( size, -size,  size);
		// Left Face
		glNormal3d(-1, 0, 0);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-size, -size, -size);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-size, -size,  size);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-size,  size,  size);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-size,  size, -size);
	glEnd ();
}
